"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


import re

meta = {
	'name': 'Stackoverflow Search',
	'author': 'Sanjiban Sengupta',
	'version': '0.5',
	'description': 'Search your query in the stackoverflow.com and show the results.',
	'sources': ('google', 'carrot2', 'bing', 'yahoo', 'millionshort', 'qwant', 'duckduckgo'),
	'options': (
		('query', None, True, 'Query string', '-q', 'store', str),
		('limit', 1, False, 'Search limit(number of pages, default=1)', '-l', 'store', int),
		('count', 50, False, 'Number of results per page(min=10, max=100, default=50)', '-c', 'store', int),
		('thread', 2, False, 'The number of engine that run per round(default=2)', '-t', 'store', int),
		('engine', 'google', False, 'Engine names for search(default=google)', '-e', 'store', str),
	),
	'examples': ('stackoverflow -q "syntax error" -l 15 --output',)
}

LINKS = []
PAGES = ''

def search(self, name, q, q_formats, limit, count):
	global PAGES,LINKS
	engine = getattr(self, name)
	q = q_formats[f"{name}_q"] if f"{name}_q" in q_formats else q_formats['default_q']
	varnames = engine.__init__.__code__.co_varnames
	if 'limit' in varnames and 'count' in varnames:
		attr = engine(q, limit, count)
	elif 'limit' in varnames:
		attr = engine(q, limit)
	else:
		attr = engine(q)

	attr.run_crawl()
	LINKS += attr.links
	PAGES += attr.pages

def module_api(self):
	query = self.options['query']
	limit = self.options['limit']
	count = self.options['count']
	engine = self.options['engine'].split(',')
	output = {'links': [], 'profiles': [], 'tags': []}
	q_formats = {
		'default_q': f"site:www.stackoverflow.com {query}",
		'millionshort_q': f'site:www.stackoverflow.com "{query}"'
	}

	self.thread(search, self.options['thread'], engine, query, q_formats, limit, count, meta['sources'])
	links = list(self.reglib().filter(r'https?://(www\.)?stackoverflow\.com', list(set(LINKS))))
	for link in links:
		first_type = re.search(r'stackoverflow\.com/[Qq]uestions/[\d]*/([\w\d\-_]+)/?', link)
		if first_type:
			title = first_type.group(1).replace('-', ' ').title()
			title = self.urlib(title).unquote.title()
		else:
			title = 'Matching Result [Without Title]'
		output['links'].append([link, title])

	for link in links:
		link = re.sub(r'https?://(www\.)stackoverflow\.com/users/', '', link)
		if re.search(r'^[\w\d_\-\/]+$', link):
			link = link.rsplit('/')
			output['profiles'].append(link[0])

	for link in links:
		if '/tags/' in link:
			link = re.sub(r'https?://(www\.)?stackoverflow\.com/tags/', '', link)
			if re.search(r'^[\w\d_\-]+$', link):
				output['tags'].append(f"#{link}")
	self.save_gather(output, 'search/stackoverflow', query, output=self.options.get('output'))
	return output

def module_run(self):
	output = module_api(self)
	for i in range(len(output['links'])):
		self.output(f"{output['links'][i][1]}")
		self.output(f"\t{output['links'][i][0]}", 'G')
	output.pop('links')
	self.alert_results(output)



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

class main:
	def __init__(self, q, limit=1, count=10):
		""" pastebin link
		q       : The query domain
		limit   : Number of pages
		"""
		self.framework = main.framework
		self.q = q.split('_')[0]
		self._pages = ''
		self.sources = ['google', 'bing', 'yahoo', 'ask', 'carrot2', 'duckduckgo', 'millionshort', 'qwant']
		self._domains = []
		self.extract_url = 'https://pastebin.com/raw'
		self.limit = limit
		self.count = count
		self.thread = 3
		self._links = []
		self.q_formats = {
			'default_q': f'site:pastebin.com "{self.q}"',
			'yippy_q': f'"pastebin.com" {self.q}',
			'qwant_q': f'site:pastebin.com {self.q}'
		}
		

	def search(self, self2, name, q, q_formats, limit, count):
		engine = getattr(self.framework, name)
		q = self.q_formats[f"{name}_q"] if f"{name}_q" in self.q_formats else self.q_formats['default_q']
		varnames = engine.__init__.__code__.co_varnames
		if 'limit' in varnames and 'count' in varnames:
			attr = engine(q, limit, count)
		elif 'limit' in varnames:
			attr = engine(q, limit)
		else:
			attr = engine(q)

		attr.run_crawl()
		self._links += attr.links
		self._pages += attr.pages

	def run_crawl(self):
		self.framework.thread(self.search, self.thread, self.sources, self.q, self.q_formats, self.limit, self.count, self.sources)

	@property
	def pages(self):
		return self._pages

	@property
	def links(self):
		return self._links

	@property
	def dns(self):
		return self.framework.page_parse(self._pages).get_dns(self.q)

	@property
	def emails(self):
		return self.framework.page_parse(self._pages).get_emails(self.q)
